"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var axios_1 = require("axios");
var querystring = require("querystring");
var QiitaError_1 = require("../errors/QiitaError");
var QiitaForbiddenError_1 = require("../errors/QiitaForbiddenError");
var QiitaInternalServerError_1 = require("../errors/QiitaInternalServerError");
var QiitaNotFoundError_1 = require("../errors/QiitaNotFoundError");
var QiitaRateLimitError_1 = require("../errors/QiitaRateLimitError");
var QiitaUnauthorizedError_1 = require("../errors/QiitaUnauthorizedError");
var QiitaURLResolveError_1 = require("../errors/QiitaURLResolveError");
var Gateway = /** @class */ (function () {
    /**
     * @param options オプショナルなパラメーター
     * @param options.url Qiitaのホストです
     * @param options.version APIバージョンを示すパスです
     * @param options.token ユーザーのアクセストークンです
     */
    function Gateway(options) {
        var _this = this;
        /** Qiitaのホストです */
        this.url = 'https://qiita.com';
        /** ユーザーのアクセストークンです */
        this.token = '';
        /** APIバージョンを示すパスです */
        this.version = '/api/v2';
        /**
         * APIクライアントに設定されたトークンを返します
         * トークンの取得を行うには `Qiita.fetchAccessToken` をご利用ください。
         * @return 設定されたトークン
         */
        this.getToken = function () { return _this.token; };
        /**
         * APIクライアントに設定されたURLを返します
         * @return 設定されたURL
         */
        this.getUrl = function () { return _this.url; };
        /**
         * APIクライアントに設定されたバージョンを示すパスを返します
         * @return 設定されたパス
         */
        this.getVersion = function () { return _this.version; };
        if (options && options.url) {
            this.url = options.url;
        }
        if (options && options.version) {
            this.version = options.version;
        }
        if (options && options.token) {
            this.token = options.token;
        }
    }
    /**
     * Qiita APIにアクセスするためのトークンを設定します
     * @param token トークン文字列
     * @return 何も返しません
     */
    Gateway.prototype.setToken = function (token) {
        this.token = token;
    };
    /**
     * QiitaへのURLを設定します
     * @param url Qiitaのホスト
     * @return 何も返しません
     */
    Gateway.prototype.setUrl = function (url) {
        // トレーリングスラッシュを削除
        this.url = url.replace(/\/$/, '');
    };
    /**
     * Qiita APIへのパスを指定します．
     * @param version APIへのパスの文字列 (e.g. `/api/v2`)
     * @return 何も返しません
     */
    Gateway.prototype.setVersion = function (version) {
        // トレーリングスラッシュを削除
        this.version = version.replace(/\/$/, '');
    };
    /**
     * Fetch APIのラッパー関数です
     * @param url リクエストするURLです
     * @param options Fetch APIの第二引数に渡されるオプションです
     * @return パースされたJSONオブジェクトを解決するPromiseです
     */
    Gateway.prototype.request = function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var error_1, _a, message, status_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!options.headers) {
                            options.headers = {};
                        }
                        if (!options.headers['Content-Type']) {
                            options.headers['Content-Type'] = 'application/json';
                        }
                        if (!this.url) {
                            throw new QiitaURLResolveError_1.QiitaURLResolveError('Qiitaのホストが指定されていません。`Qiita.setUrl` でAPIのホストを指定してからメソッドを呼び出してください。');
                        }
                        if (this.token) {
                            options.headers.Authorization = "Bearer " + this.token;
                        }
                        options.transformResponse = [function (data) {
                                try {
                                    return JSON.parse(data);
                                }
                                catch (_a) {
                                    return data;
                                }
                            }];
                        _b.label = 1;
                    case 1:
                        _b.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, axios_1.default.request(options)];
                    case 2: return [2 /*return*/, _b.sent()];
                    case 3:
                        error_1 = _b.sent();
                        _a = error_1.response, message = _a.message, status_1 = _a.status;
                        switch (status_1) {
                            case 401:
                                throw new QiitaUnauthorizedError_1.QiitaUnauthorizedError(message);
                            case 403:
                                throw new QiitaForbiddenError_1.QiitaForbiddenError(message);
                            case 404:
                                throw new QiitaNotFoundError_1.QiitaNotFoundError(message);
                            case 429:
                                throw new QiitaRateLimitError_1.QiitaRateLimitError(message);
                            case 500:
                                throw new QiitaInternalServerError_1.QiitaInternalServerError(message);
                            default:
                                throw new QiitaError_1.QiitaError('QiitaError', message);
                        }
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * HTTP GETのラッパー関数です
     * @param url リクエストするURL
     * @param params クエリ文字列
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.get = function (url, params, options) {
        if (params === void 0) { params = {}; }
        if (options === void 0) { options = {}; }
        return this.request(__assign({ method: 'GET', url: url + (Object.keys(params).length ? '?' + querystring.stringify(params) : '') }, options));
    };
    /**
     * HTTP POSTのラッパー関数です
     * @param url リクエストするURL
     * @param body リクエストボディ
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.post = function (url, body, options) {
        if (body === void 0) { body = {}; }
        if (options === void 0) { options = {}; }
        return this.request(__assign({ method: 'POST', url: url, data: JSON.stringify(body) }, options));
    };
    /**
     * HTTP PUTのラッパー関数です
     * @param url リクエストするURL
     * @param body リクエストボディ
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.put = function (url, body, options) {
        if (body === void 0) { body = {}; }
        if (options === void 0) { options = {}; }
        return this.request(__assign({ method: 'PUT', url: url, data: JSON.stringify(body) }, options));
    };
    /**
     * HTTP DELETEのラッパー関数です
     * @param url リクエストするURL
     * @param body リクエストボディ
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.delete = function (url, body, options) {
        if (body === void 0) { body = {}; }
        if (options === void 0) { options = {}; }
        return this.request(__assign({ method: 'DELETE', url: url, data: JSON.stringify(body) }, options));
    };
    /**
     * HTTP PATCHのラッパー関数です
     * @param url リクエストするURL
     * @param body リクエストボディ
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.patch = function (url, body, options) {
        if (body === void 0) { body = {}; }
        if (options === void 0) { options = {}; }
        return this.request(__assign({ method: 'PATCH', url: url, data: JSON.stringify(body) }, options));
    };
    return Gateway;
}());
exports.Gateway = Gateway;
//# sourceMappingURL=Gateway.js.map